<?php

	/* Alldebrid host module for synology download station */
	/* original by NutTom, modified version with new API and updated domain list by Alld Staff */
	/* Version 1.0 */
	
	// Partially inspired from:
	// https://github.com/TheNitek/synoOpenload/blob/master/host.php
	// https://github.com/mcampbell508/Synology-DownloadStation-Hosts/blob/master/oteupload/host.php
	
	// 2016-01-23:  - Better error handle, with retry later
	//				- Two type of URL catch
	//				- debug to tmp file
	

	class SynoFileHostingAllDebrid
	{
		private $Url;
		private $Username;
		private $Password;
		private $HostInfo;

		private $_version = '2.1';
		
		private $_tokenPath = '/tmp/alldebrid.token';

		public function __construct($Url, $Username, $Password, $HostInfo)
		{
			$this->Url = $Url;
			$this->Username = $Username;
			$this->Password = $Password;
			$this->HostInfo = $HostInfo;
		}
		
		// This function check if the account is valid
		public function Verify($ClearCookie)
		{	
			//Handle force clear of auth data
			if($ClearCookie) {
				$token = '';
				file_put_contents($this->_tokenPath, $token);
			} else {
				$token = file_get_contents($this->_tokenPath);
			}

			//Try logging with token
			if($token AND strlen($token) > 32) {
				$status = $this->alldebridLoginByToken($token);

				//If login failed, let' try to relogin with username/password
				if($status != LOGIN_FAIL)
					return $status;
			}

			// Logging with user credentials
			if(!empty($this->Username) && !empty($this->Password)) {
				return $this->alldebridLogin($this->Username, $this->Password);
			}
			
			return LOGIN_FAIL;
		}
	
		// This function perform logging to get a valid session (cookie)
		private function alldebridLogin($username, $password)
		{						
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($curl, CURLOPT_USERAGENT, DOWNLOAD_STATION_USER_AGENT);
			// curl_setopt($curl, CURLOPT_HEADER, TRUE);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_URL, 'https://api.alldebrid.com/user/login?agent=synology&version=' . $this->_version . '&username=' . urlencode($username) . '&password=' . urlencode($password));
			$LoginInfo = curl_exec($curl);
			curl_close($curl);

			$userInfos = json_decode($LoginInfo, true);

			if(isset($userInfos['success']) AND $userInfos['success'] == true) {
				if($userInfos['user']['isPremium'] == true) {
					file_put_contents($this->_tokenPath, $userInfos['token']);
					return USER_IS_PREMIUM;
				}
				else
					return USER_IS_FREE;
			} else 
				return LOGIN_FAIL;
		}

		// This function perform logging to get a valid session (cookie)
		private function alldebridLoginByToken($token)
		{						
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($curl, CURLOPT_USERAGENT, DOWNLOAD_STATION_USER_AGENT);
			// curl_setopt($curl, CURLOPT_HEADER, TRUE);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_URL, 'https://api.alldebrid.com/user/login?agent=synology&version=' . $this->_version . '&token=' . urlencode($token));
			$LoginInfo = curl_exec($curl);
			curl_close($curl);

			$userInfos = json_decode($LoginInfo, true);

			if(isset($userInfos['success']) AND $userInfos['success'] == true) {
				if($userInfos['user']['isPremium'] == true) {
					return USER_IS_PREMIUM;
				}
				else
					return USER_IS_FREE;
			} else 
				return LOGIN_FAIL;
		}
		
		//This function returns download url.
		public function GetDownloadInfo()
		{	
			if($this->Verify(false) != USER_IS_PREMIUM) {
				$downloadInfo = array();
				$downloadInfo[DOWNLOAD_ERROR] = ERR_REQUIRED_PREMIUM;

				return $downloadInfo;
			}

			$token = file_get_contents($this->_tokenPath);

			$downloadInfo = array();
			$downloadInfo[DOWNLOAD_ISPARALLELDOWNLOAD] = TRUE;
		
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($curl, CURLOPT_USERAGENT, DOWNLOAD_STATION_USER_AGENT);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_URL, 'https://api.alldebrid.com/link/unlock?agent=synology&version=' . $this->_version . '&token=' . urlencode($token) . '&link=' . urlencode($this->Url));
			$response = curl_exec($curl);
			curl_close($curl);

			$response = json_decode($response, true);

			if(isset($response['success']) AND $response['success'] == true) {
				$downloadInfo[DOWNLOAD_URL] = $response['infos']['link'];
			} else {
				switch ($response['errorCode']) {
					case 30:
						$downloadInfo[DOWNLOAD_ERROR] = ERR_NOT_SUPPORT_TYPE;
						break;
					case 31:
						$downloadInfo[DOWNLOAD_ERROR] = ERR_FILE_NO_EXIST;
						break;
					case 32:
					case 33:
					case 34:
					case 35:
					case 36:
						$downloadInfo[DOWNLOAD_ERROR] = ERR_TRY_IT_LATER;
						break;
					default:
						$downloadInfo[DOWNLOAD_ERROR] = ERR_UNKNOWN;
						break;
				}
			}
			
			return $downloadInfo;
		}
		
		private function debug($Header, $Value)
		{
			// $msg = $Header . print_r($Value, TRUE) . "\n";   
			// if (DEBUG_TO_FILE)
			// {
			// 	$msg = date("n-d H:i:s") . " " . $msg;
			// 	file_put_contents(DEBUG_FILE, $msg, FILE_APPEND);   
			// }
		}
	}
?>